;================================== Credits =====================================
;Language       FreeBASIC 0.20 for Linux
;Copyright © 2009 David Watson [david@watzon.co.uk]

;Anyone is permitted to copy, modify, distribute and/or sell this program,
;provided that the copyright notice and permission notices appear in all copies.

;Anyone is permitted to use this program, provided that the user accept
;the program is provided "as is" and comes without any warranty.

;Anyone is permitted to integrate parts of the source code of this program into
;their own, provided that the author of this program is duly credited.
;================================================================================
;Language:   BlitzPlus on Windows 10
;Copyrighe (c) 2021 Andres Amaya Jr

;All of the above permissions and restrictions applies to this code as well.

;This version is mostly the same but with some nice speed optimizations.

;Addition of ease of use features in a graphical user interface (GUI).
;(Color choices, upscaled plot size, save and load features, etc...) .
;================================================================================
	

AppTitle "Petri Dish"

Global dlaSize%, vsArray%, circRad%, mag%, diskFile$
Global picFlag%, pix%, bak%, rbs%, typed%


;========================= Initialize screen and variables ======================
Graphics 812, 560, 32, 2
SetBuffer BackBuffer()
SeedRnd MilliSecs()


Dim vs%(511,511)
Dim dx%(999), dy%(999), px%(999), py%(999)
Dim pal%(17), p3%(17,2)
palet()

dlaSize = 128

Local splash%, altSkin%, skin%, st%, mx%, my%
splash = LoadImage(".\img\Gui.png")
rbs = LoadImage(".\img\radioButtonSelected.png")
typed = LoadSound(".\img\typewriter-key-1.wav")

;================================================================================
;                Select Alternate GUI skin here
;================================================================================
altSkin = 3
;================================================================================

If altSkin > 0 Then
	Select altSkin
		Case 1: skin = LoadImage(".\img\Blue skin panel.png")
		Case 2: skin = LoadImage(".\img\DarkBlue skin panel.png")
		Case 3: skin = LoadImage(".\img\DarkGreen skin panel.png")
		Case 4: skin = LoadImage(".\img\DarkRed skin panel.png")
		Case 5: skin = LoadImage(".\img\Green skin panel.png")
	End Select
End If
DrawBlock splash, 0, 0
If altSkin > 0 Then DrawBlock skin, 560, 0
Flip
WaitMouse()

pix = pal(0)	;Default DLA pixel drawing color
bak = pal(11)	;Default background color


While exitFlag = 0
	If MouseHit(1)
		mx = MouseX()
		my = MouseY()
		;DebugLog "MouseX = " + mx + "  MouseY = "+my
		;Check Radio Buttons
		If pnr(mx, my, 594, 24, 161, 15) Then PlaySound typed: rbZone(mx, my)
		;Check Background color selection
		If pnr(mx, my, 575,  74, 154, 75) Then PlaySound typed: setBackColor(mx, my)
		;Check Foreground color selection
		If pnr(mx, my, 575, 182, 154, 75) Then PlaySound typed: setForeColor(mx, my)
		;Check if a 'Button' has been pressed
		If pnr(mx, my, 569, 274, 234, 275) Then
			buttonPressed = whichButton(mx, my)
			Select buttonPressed
				Case 1
					;Generate a new DLA
					PlaySound typed: genDLA()
				Case 2
					;Re-Color a new or loaded DLA
					PlaySound typed: reColorDLA()
				Case 3
					;Save a DLA map to disk
					PlaySound typed: saveMap()
				Case 4
					;Load a saved DLA map from disk
					PlaySound typed
					;================================================
					showExample = 3 ;<------ Select map to load here
					;================================================
					Select showExample
						Case 1	;this is a 128x128 DLA
							loadMap("DLA_128_20210926_23_07_20.txt") 
						Case 2	;this is a 256x256 DLA
							loadMap("DLA_256_20210925_16_45_13.txt")
						Case 3	;this is a 512x512 DLA
							loadMap("DLA_512_20210926_17_46_19.txt")
					End Select
				Case 5
					;Save DLA to BMP
					PlaySound typed: saveBMP()
				Case 6
					;Bring up the 'Walk Through' html page  (NOT working)
					file$ = "Petri Dish Walk Through.htm"
					ExecFile(file$)
				Case 7
					PlaySound typed
					Delay 500
					;Terminate and free up memory
					If altSkin <> 0 Then FreeImage skin
					FreeImage splash
					FreeImage rbs
					FreeSound typed
					End
			End Select
		End If
	End If
	FlushMouse()
	If KeyHit(1) Then exitFlag = 1
	Delay 30	;lower CPU usage
Wend

If exitFlage = 1 Then
	;Terminate and free up memory when [ESC] key pressed
	FreeImage splash
	FreeImage rbs
	FreeSound typed
	End
End If

Function rbZone(rbx%, rby%)
	dColor($FFFFFF)
	If pnc(rbx, rby, 602, 31, 9) Then
		Rect(671, 28, 7, 7, True)
		Rect(743, 28, 7, 7, True)
		DrawBlock rbs, 599, 28
		dlaSize = 128
	End If
	If pnc(rbx, rby, 674, 31, 9) Then
		Rect(599, 28, 7, 7, True)
		Rect(743, 28, 7, 7, True)
		DrawBlock rbs, 671, 28
		dlaSize = 256
	End If
	If pnc(rbx, rby, 746, 31, 9) Then
		Rect(599, 28, 7, 7, True)
		Rect(671, 28, 7, 7, True)
		DrawBlock rbs, 743, 28
		dlaSize = 512
	End If
	dColor(pix)
	Flip
End Function

Function setBackColor(bcx%, bcy%)
    If pnr(bcx, bcy, 576, 75, 23, 23) Then bak = pal(0)
    If pnr(bcx, bcy, 601, 75, 23, 23) Then bak = pal(1)
    If pnr(bcx, bcy, 627, 75, 23, 23) Then bak = pal(2)
    If pnr(bcx, bcy, 653, 75, 23, 23) Then bak = pal(3)
    If pnr(bcx, bcy, 679, 75, 23, 23) Then bak = pal(4)
    If pnr(bcx, bcy, 705, 75, 23, 23) Then bak = pal(5)
    If pnr(bcx, bcy, 576, 100, 23, 23) Then bak = pal(6)
    If pnr(bcx, bcy, 601, 100, 23, 23) Then bak = pal(7)
    If pnr(bcx, bcy, 627, 100, 23, 23) Then bak = pal(8)
    If pnr(bcx, bcy, 653, 100, 23, 23) Then bak = pal(9)
    If pnr(bcx, bcy, 679, 100, 23, 23) Then bak = pal(10)
    If pnr(bcx, bcy, 705, 100, 23, 23) Then bak = pal(11)
    If pnr(bcx, bcy, 576, 125, 23, 23) Then bak = pal(12)
    If pnr(bcx, bcy, 601, 125, 23, 23) Then bak = pal(13)
    If pnr(bcx, bcy, 627, 125, 23, 23) Then bak = pal(14)
    If pnr(bcx, bcy, 653, 125, 23, 23) Then bak = pal(15)
    If pnr(bcx, bcy, 679, 125, 23, 23) Then bak = pal(16)
    If pnr(bcx, bcy, 705, 125, 23, 23) Then bak = pal(17)
	dColor(bak)
	Rect(740, 75, 51, 72, True)
	dColor(pix)
	Flip
End Function

Function setForeColor(fcx%, fcy%)
    If pnr(fcx, fcy, 576, 183, 23, 23) Then pix = pal(0)
    If pnr(fcx, fcy, 601, 183, 23, 23) Then pix = pal(1)
    If pnr(fcx, fcy, 627, 183, 23, 23) Then pix = pal(2)
    If pnr(fcx, fcy, 653, 183, 23, 23) Then pix = pal(3)
    If pnr(fcx, fcy, 679, 183, 23, 23) Then pix = pal(4)
    If pnr(fcx, fcy, 705, 183, 23, 23) Then pix = pal(5)
    If pnr(fcx, fcy, 576, 208, 23, 23) Then pix = pal(6)
    If pnr(fcx, fcy, 601, 208, 23, 23) Then pix = pal(7)
    If pnr(fcx, fcy, 627, 208, 23, 23) Then pix = pal(8)
    If pnr(fcx, fcy, 653, 208, 23, 23) Then pix = pal(9)
    If pnr(fcx, fcy, 679, 208, 23, 23) Then pix = pal(10)
    If pnr(fcx, fcy, 705, 208, 23, 23) Then pix = pal(11)
    If pnr(fcx, fcy, 576, 233, 23, 23) Then pix = pal(12)
    If pnr(fcx, fcy, 601, 233, 23, 23) Then pix = pal(13)
    If pnr(fcx, fcy, 627, 233, 23, 23) Then pix = pal(14)
    If pnr(fcx, fcy, 653, 233, 23, 23) Then pix = pal(15)
    If pnr(fcx, fcy, 679, 233, 23, 23) Then pix = pal(16)
    If pnr(fcx, fcy, 705, 233, 23, 23) Then pix = pal(17)
	dColor(pix)
	Rect(740, 183, 51, 72, True)
	Flip
End Function

Function whichButton(whx%, why%)
	Local choice%
	choice = 0
    If pnr(whx, why, 570, 275, 232, 23) Then choice = 1
    If pnr(whx, why, 570, 307, 232, 23) Then choice = 2
    If pnr(whx, why, 570, 353, 232, 23) Then choice = 3
    If pnr(whx, why, 570, 383, 232, 23) Then choice = 4
    If pnr(whx, why, 570, 413, 232, 23) Then choice = 5
    If pnr(whx, why, 570, 469, 232, 23) Then choice = 6
    If pnr(whx, why, 570, 525, 232, 23) Then choice = 7
	Return choice
End Function
	
Function genDLA ()
	Local ct%, counter%, modVal%, finished%, bMax%, st%
	Local a%, d%, r%, n%, x%, y%, q%

	;Dim vs(dlaSize - 1, dlaSize - 1)

	dColor(bak)
	Rect(0, 0, 560, 560, True)
	dColor(pix)
	Oval (24, 24, 512, 512, False)

	If dlaSize = 128 Then mag = 4
	If dlaSize = 256 Then mag = 2
	If dlaSize = 512 Then mag = 1

	st = MilliSecs()
	;===============================================================================
	;	Initialize Variables and Temporary Dots
	;===============================================================================
	;Set vs() array to all zeros
	For y = 0 To 511
		For x = 0 To 511
			vs(x,y) = 0
		Next
	Next
	
	counter = 1
	modVal = 1000
	finished = 0
	bMax = dlaSize - 1
	r = 1
	ct = dlaSize Shr 1
	Plot (280, 280)		;draw pix dot at center of DLA screen
	vs(ct, ct) = 1		;mark pix dot at center of vs() array
	Flip

	For d = 0 To 999
   		;temp dots start in disc around centre
   		a = Rand (0,359)
   		r = Rand(7,56)
   		px(d) = r * Sin(a) + ct
   		py(d) = r * Cos(a) + ct
   		;temp dots move up, down, left or right but are not stationary
   		Repeat
       		dx(d) = Rand(-1, 1)
       		dy(d) = Rand(-1, 1)
   		Until dx(d) <> 0 Or dy(d) <> 0
	Next
	;===============================================================================
	;	Brownian Motion Loop
	;===============================================================================
	While finished = 0

		LockBuffer ;BackBuffer()

    	For d = 0 To 999
        	;see if temp dot has any pix dot neighbours
       		n = 0
        	For y = -1 To 1
            	For x = -1 To 1
					;If there's a pix dot in one of the 8 surrounding pixels THEN set 'n flag' to 1
					;posX% = (px(d) + x) And bMax : posY% = (py(d) + y) And bMax
					If vs( (px(d) + x) And bMax, (py(d) + y) And bMax) = 1 Then ;n = 1
						n = 1
						;DebugLog "vs("+x+", "+y+") = " + posX% +", "+ posY%
						Exit
					End If
            	Next
				If n=1 Then Exit
        	Next

			;temp dot has a pix dot for neighbour
        	If n = 1 Then
				;temp dot becomes a DLA pix dot in the vs() array
				vs(px(d), py(d)) = 1	;Turn temp dot into a DLA dot
				If mag > 1 Then 
					fCirc( px(d) * mag + 24, py(d) * mag + 24, mag+2, pix)
				Else
					WritePixelFast(px(d) + 24, py(d) + 24, pix)
				End If
			
				counter = counter + 1	;count number of DLA pixels plotted
				If counter Mod modVal = 0 Then
					UnlockBuffer BackBuffer()
					Flip
					LockBuffer BackBuffer()
				End If

            	;check to see if structure has gotten bigger
            	x = Abs(px(d) - ct)
            	y = Abs(py(d) - ct)
            	q = Sqr(x * x + y * y)
            	If q > r Then r = q


            	;new temp dot placed away from structure's edge
            	a = Rand (0,359)
           		px(d) = (r + 4) * Cos(a) + ct
           		py(d) = (r + 4) * Sin(a) + ct

        	Else ;temp dot doesn't have pix dot for neighbour

				;erase temp dot in the vs() array
				vs( px(d), py(d) ) =  0

            	If Rand(0,100) < 10 Then ;randomly change dot heading
                	Repeat
                    	dx(d) = Rand(-1, 1)
                    	dy(d) = Rand(-1, 1)
                	Until dx(d) <> 0 Or dy(d) <> 0
           		End If

            	;move temp dots
            	x = Abs(px(d) + dx(d) - ct)
            	y = Abs(py(d) + dy(d) - ct)
            	q = Sqr(x * x + y * y)

            	If (q - r) < 7 Then
                	px(d) = (px(d) + dx(d)) And bMax
                	py(d) = (py(d) + dy(d)) And bMax
            	End If
        	End If


			px(d) = px(d) And bMax
			py(d) = py(d) And bMax

			;place a temp dot in the vs() array 
			vs( px(d), py(d) ) = 2       
    	Next

		UnlockBuffer

		;halt program if structure has grown to maximum size
    	If r > (ct-4) Then 

			;erase all temp dots from vs() array
        	For d = 0 To 999
				vs( px(d), py(d) ) = 0
        	Next

			Text 5,2,"Done in "+(MilliSecs()-st) +"ms"
			Text 400, 2, counter + " DLA dots"

			Flip

			;prepare file name that can be use for DLA map file or save to .BMP
			date$ = CurrentDate()	;returns DD MMM YYYY
			month$ = Mid(date$, 4, 3)
			allMonth$ = "   Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec"
			newMonth = Instr(allMonth$, month$)
			digiMonth$ =Right("0"+ Str(newMonth/4),2)
			;reformatted to YYYMMMDD for sorting
			tod$ = Right(date$,4)+digiMonth$+Left(date$,2)

			time$ = CurrentTime()	;returns HH:MM:SS (24 hour format) 
			;reformatted to HH_MM_SS with valid file name characters (underscores)
			time$ = Left(time$,2)+"_"+Mid(time$,4,2)+"_"+Right(time$,2)

			;filename suitable for output
			diskFile$ = "DLA_"+Str(dlaSize)+"_"+tod$+"_"+time

			vsArray = dlaSize
			finished = 1		
    	End If
    	If KeyHit(1) Then End
	Wend
End Function

Function reColorDLA ()
	Local x%, y%, counter%
    If vsArray = dlaSize Then
		dColor(bak) : Rect(0, 0, 560, 560, True) ;Set DLA screen to background color
		dColor(pix) : Oval (24, 24, 512, 512, False) ;Draw the petri dish with foreground color
		LockBuffer()
	    ;Show the graphic contents of the virtual screen array
        For y = 0 To dlaSize - 1
            For x = 0 To dlaSize - 1
                If vs(x, y) = 1 Then
                    If mag > 1 Then
                        fCirc(x * mag + 24, y * mag + 24, mag + 2, pix)
                    Else
						WritePixelFast( x + 24, y + 24, pix)
                    End If
                    counter = counter + 1
                End If
            Next
        Next
		UnlockBuffer()
        ;Show number of DLA Dots in the Re-Colored map array
        Text 360, 0, counter + " DLA dots"
		Flip
    Else
		dColor(bak): Rect(0, 544, 560, 16, True): dColor(pix)
        Text 2, 545, ">>> You must generate, or load DLA before ReColor <<<"
		Flip
    End If
End Function

Function saveMap ()
	Local x%, y%, fileOut%
	Local fileName$, accumulate$
 
	fileName = diskFile + ".txt"
	fileOut = WriteFile(fileName)
		
	WriteLine(fileOut, "Map Size = " + Str(dlaSize) )
	For y = 0 To dlaSize - 1
		accumulate = ""
		For x = 0 To dlaSize - 1
			accumulate = accumulate + Str(vs(x,y) )
		Next
		WriteLine(fileOut, accumulate)
	Next
	CloseFile fileOut
	dColor(bak) : Rect(0, 544, 560, 16, True) : dColor(pix)
	Text (2, 545, ">>> " + fileName  +  " <<<    Saved.")
	Flip
End Function


Function loadMap%(mapName$)
	Local counter%, x%, y%, fileIn%
	Local fileName$, sz$, temp$
	
	fileName = mapName$
	
	fileIn = ReadFile(fileName)
	temp$ = ReadLine$(fileIn)
	sz = Right$(temp,3)
	
	DebugLog "sz = " + sz
	
	;Update the radio buttons to reflect the size of loaded DLA map
	dColor($FFFFFF)
	Rect(599, 28, 7, 7, True)
	Rect(671, 28, 7, 7, True)
	Rect(743, 28, 7, 7, True)
	Select sz
		Case "128"
			dlaSize = 128 : mag = 4
			DrawBlock rbs, 599, 28
		Case "256"
			dlaSize = 256 : mag = 2
			DrawBlock rbs, 671, 28
		Case "512"
			dlaSize = 512 : mag = 1
			DrawBlock rbs, 743, 28
	End Select
	vsArray = dlaSize
	Flip
	;For y = 0 To dlaSize - 1
	;	For x = 0 To dlaSize - 1
	;		vs(x,y) = 0
	;	Next
	;Next
	;Clear the DLA screen with background color
	dColor(bak)
	Rect(0, 0, 560, 560, True)
	;Draw the petri dish with foreground color
	dColor(pix)
	Oval (24, 24, 512, 512, False)
	Flip

	LockBuffer()
	For y = 0 To dlaSize - 1
		temp$ = ReadLine$(fileIn)
		For x = 0 To dlaSize - 1
			If Mid(temp$, x + 1, 1) = "1" Then
				vs(x, y) = 1
				If mag > 1 Then
					fCirc(x * mag + 24, y * mag + 24, mag + 2, pix)
				Else
					WritePixelFast(x + 24, y + 24, pix)
				End If
				counter = counter + 1
			Else
				vs(x, y) = 0
			End If
		Next
	Next
	UnlockBuffer()

	CloseFile fileIn
	diskFile = Left(fileName, Len(fileName) - 4)
	dColor(bak) : Rect(0, 544, 560, 16, True) : dColor(pix)
	;Show the number of DLA dots in the retrieved map array
	Text 360, 0, counter + " DLA dots"
	Text 2, 545, ">>> " + fileName + " <<< loaded."
	Flip
End Function

Function saveBMP ()
	Local pic%, fileName$
	
	fileName = diskFile + ".bmp"
	pic = CreateImage(560, 560)
	GrabImage pic, 0, 0
	
	SaveImage(pic,fileName)
	dColor(bak) : Rect(0, 544, 560, 16, True) : dColor(pix)
	Text 2, 545, ">>> " + fileName  +  " <<<    Saved."
	Flip
	;Notify "Save BMP" + Chr$(13) + filename + Chr$(13) +" saved to disk."
	FreeImage pic
End Function

Function palet ()
	Local i%
    ;This is the place where you can change the Foreground and Background Colors
    ;If you make any changes they will NOT show up in the GUI unless you go into
    ;your favorite image editor and also change the "GUI.png" in the 'img' folder

    pal(0)  = $FF000000 ;Black
    pal(1)  = $FF888890 ;Gray
    pal(2)  = $FF880015 ;Dark Red
    pal(3)  = $FFFF0000 ;Red
    pal(4)  = $FFFF7000 ;Orange
    pal(5)  = $FFFFFF00 ;Yellow
    pal(6)  = $FF00FF00 ;Green
    pal(7)  = $FF005000 ;Dark Green
    pal(8)  = $FF4080FF ;Light Blue
    pal(9)  = $FF0000FF ;Blue
    pal(10) = $FF6000A0 ;Purple
    pal(11) = $FFFFFFFF ;White
    pal(12) = $FF402010 ;Brown
    pal(13) = $FFFFC0A0 ;Tan
    pal(14) = $FFA0FFC0 ;Light Green
    pal(15) = $FF000080 ;Dark Blue
    pal(16) = $FFFF00FF ;Magenta
    pal(17) = $FFB0B0B0 ;Light Gray
End Function

Function dColor (hxClr%)
	Local r%, g%, b%
	r = (hxClr Shr 16) And 255	;Red value
	g = (hxClr Shr 8) And 255	;Green value
	b = hxClr And 255			;Blue value
	Color r, g, b
End Function

Function pnr (pntX, pntY, rectX, rectY, rectW, rectH)
    ;=========================================================================as  ===========
    ;   Function "Point iN Rectangle"
    ;====================================================================================
    ; This function checks to see if the point (px,py) is within the specified rectangle.
    ;
    ; If the point is inside the rectangle a value of -1 is returned.
    ;
    ; If the point is not inside the rectangle a value of 0 (zero) is returned.
    ;====================================================================================
    ; pntX  = X coord of the point to check
    ; pntY  = Y coord of the point to check
    ; rectX = upper left X coord of rectangle
    ; rectY = upper left Y coord of rectangle
    ; rectW = width of rectangle
    ; rectH = height of rectangle
    ;====================================================================================
    Return ((pntX >= rectX) And (pntX <= (rectX + rectW - 1)) And (pntY >= rectY) And (pntY <= (rectY + rectH - 1)))
End Function

Function pnc (pntX, pntY, cntrX, cntrY, circR)
    ;====================================================================================
    ;   Function "Point iN Circle"
    ;====================================================================================
    ; This function checks to see if the point (px,py) is inside the specified circle.
    ;
    ; If the point is inside the circle a value of -1 is returned.
    ;
    ; If the point is not inside the circle a value of 0 (zero) is returned.
    ;====================================================================================
    ; pntX = the X coord of the point to check
    ; pntY = the Y coord of the point to check
    ; cntrX = center of circle X coord
    ; cntrY = center of circle Y coord
    ; circR = radius of circle
    ;
    ;====================================================================================
    Return (((pntX - cntrX) * (pntX - cntrX) + (pntY - cntrY) * (pntY - cntrY)) <= circR * circR)
End Function

Function fCirc (CX%, CY%, R%, hxClr%, fill% = 1)
;=============================================================
; This is one fuuugly function but it is - very - quick
; (it only draws filled circles when the buffer is locked)
;=============================================================

	;CX plus one, plus two, and plus three
	Local cxp1% = CX+1, cxp2% = CX+2, cxp3% = CX+3
	;CY plus one, plus two, and plus three
	Local cyp1% = CY+1, cyp2% = CY+2, cyp3% = CY+3
	
	;CX minus one, minus two, and minus three
	Local cxm1% = CX-1, cxm2% = CX-2, cxm3% = CX-3
	;CY minus one, minus two, and minus three
	Local cym1% = CY-1, cym2% = CY-2, cym3% = CY-3
	
	If R < 8 Then
		Select R
			Case 2
				WritePixelFast(  CX,   CY, hxClr )	
				WritePixelFast(cxp1,   CY, hxClr )	
				WritePixelFast(  CX, cyp1, hxClr )	
				WritePixelFast(cxp1, cyp1, hxClr )	
			Case 3
				WritePixelFast(cxm1, cym1, hxClr )
				WritePixelFast(  CX, cym1, hxClr )
				WritePixelFast(cxp1, cym1, hxClr )
			
				WritePixelFast(cxm1,   CY, hxClr )
				WritePixelFast(  CX,   CY, hxClr )
				WritePixelFast(cxp1,   CY, hxClr )
			
				WritePixelFast(cxm1, cyp1, hxClr )
				WritePixelFast(  CX, cyp1, hxClr )
				WritePixelFast(cxp1, cyp1, hxClr )
			Case 4
				WritePixelFast(  CX, cym1, hxClr )
				WritePixelFast(cxp1, cym1, hxClr )
			
				WritePixelFast(cxm1,   CY, hxClr )
				WritePixelFast(  CX,   CY, hxClr )
				WritePixelFast(cxp1,   CY, hxClr )
				WritePixelFast(cxp2,   CY, hxClr )
			
				WritePixelFast(cxm1, cyp1, hxClr )
				WritePixelFast(  CX, cyp1, hxClr )
				WritePixelFast(cxp1, cyp1, hxClr )
				WritePixelFast(cxp2, cyp1, hxClr )
			
				WritePixelFast(  CX, cyp2, hxClr )
				WritePixelFast(cxp1, cyp2, hxClr )
			Case 5
				WritePixelFast(cxm1, cym2, hxClr )
				WritePixelFast(CX  , cym2, hxClr )
				WritePixelFast(cxp1, cym2, hxClr )
			
				WritePixelFast(cxm2, cym1, hxClr )
				WritePixelFast(cxm1, cym1, hxClr )
				WritePixelFast(CX  , cym1, hxClr )
				WritePixelFast(cxp1, cym1, hxClr )
				WritePixelFast(cxp2, cym1, hxClr )

				WritePixelFast(cxm2, CY  , hxClr )
				WritePixelFast(cxm1, CY  , hxClr )
				WritePixelFast(CX  , CY  , hxClr )
				WritePixelFast(cxp1, CY  , hxClr )
				WritePixelFast(cxp2, CY  , hxClr )

				WritePixelFast(cxm2, cyp1, hxClr )
				WritePixelFast(cxm1, cyp1, hxClr )
				WritePixelFast(CX  , cyp1, hxClr )
				WritePixelFast(cxp1, cyp1, hxClr )
				WritePixelFast(cxp2, cyp1, hxClr )

				WritePixelFast(cxm1, cyp2, hxClr )
				WritePixelFast(CX  , cyp2, hxClr )
				WritePixelFast(cxp1, cyp2, hxClr )
			Case 6
				WritePixelFast(cxm1, cym2, hxClr )
				WritePixelFast(CX  , cym2, hxClr )
				WritePixelFast(cxp1, cym2, hxClr )
				WritePixelFast(cxp2, cym2, hxClr )

				WritePixelFast(cxm2, cym1, hxClr )
				WritePixelFast(cxm1, cym1, hxClr )
				WritePixelFast(CX  , cym1, hxClr )
				WritePixelFast(cxp1, cym1, hxClr )
				WritePixelFast(cxp2, cym1, hxClr )
				WritePixelFast(cxp3, cym1, hxClr )

				WritePixelFast(cxm2, CY  , hxClr )
				WritePixelFast(cxm1, CY  , hxClr )
				WritePixelFast(CX  , CY  , hxClr )
				WritePixelFast(cxp1, CY  , hxClr )
				WritePixelFast(cxp2, CY  , hxClr )
				WritePixelFast(cxp3, CY  , hxClr )

				WritePixelFast(cxm2, cyp1, hxClr )
				WritePixelFast(cxm1, cyp1, hxClr )
				WritePixelFast(CX  , cyp1, hxClr )
				WritePixelFast(cxp1, cyp1, hxClr )
				WritePixelFast(cxp2, cyp1, hxClr )
				WritePixelFast(cxp3, cyp1, hxClr )

				WritePixelFast(cxm2, cyp2, hxClr )
				WritePixelFast(cxm1, cyp2, hxClr )
				WritePixelFast(CX  , cyp2, hxClr )
				WritePixelFast(cxp1, cyp2, hxClr )
				WritePixelFast(cxp2, cyp2, hxClr )
				WritePixelFast(cxp3, cyp2, hxClr )

				WritePixelFast(cxm1, cyp3, hxClr )
				WritePixelFast(CX  , cyp3, hxClr )
				WritePixelFast(cxp1, cyp3, hxClr )
				WritePixelFast(cxp2, cyp3, hxClr )
			Case 7
				WritePixelFast(cxm1, cym3, hxClr )
				WritePixelFast(CX  , cym3, hxClr )
				WritePixelFast(cxp1, cym3, hxClr )

				WritePixelFast(cxm2, cym2, hxClr )
				WritePixelFast(cxm1, cym2, hxClr )
				WritePixelFast(CX  , cym2, hxClr )
				WritePixelFast(cxp1, cym2, hxClr )
				WritePixelFast(cxp2, cym2, hxClr )


				WritePixelFast(cxm3, cym1, hxClr )
				WritePixelFast(cxm2, cym1, hxClr )
				WritePixelFast(cxm1, cym1, hxClr )
				WritePixelFast(CX  , cym1, hxClr )
				WritePixelFast(cxp1, cym1, hxClr )
				WritePixelFast(cxp2, cym1, hxClr )
				WritePixelFast(cxp3, cym1, hxClr )

				WritePixelFast(cxm3, CY  , hxClr )
				WritePixelFast(cxm2, CY  , hxClr )
				WritePixelFast(cxm1, CY  , hxClr )
				WritePixelFast(CX  , CY  , hxClr )
				WritePixelFast(cxp1, CY  , hxClr )
				WritePixelFast(cxp2, CY  , hxClr )
				WritePixelFast(cxp3, CY  , hxClr )

				WritePixelFast(cxm3, cyp1, hxClr )
				WritePixelFast(cxm2, cyp1, hxClr )
				WritePixelFast(cxm1, cyp1, hxClr )
				WritePixelFast(CX  , cyp1, hxClr )
				WritePixelFast(cxp1, cyp1, hxClr )
				WritePixelFast(cxp2, cyp1, hxClr )
				WritePixelFast(cxp3, cyp1, hxClr )

				WritePixelFast(cxm2, cyp2, hxClr )
				WritePixelFast(cxm1, cyp2, hxClr )
				WritePixelFast(CX  , cyp2, hxClr )
				WritePixelFast(cxp1, cyp2, hxClr )
				WritePixelFast(cxp2, cyp2, hxClr )
	
				WritePixelFast(cxm1, cyp3, hxClr )
				WritePixelFast(CX  , cyp3, hxClr )
				WritePixelFast(cxp1, cyp3, hxClr )
		End Select
		;Return
	Else
	


		Local X%, Y%, Rad%
		Local XChange%, YChange%
		Local RadiusError%
		Rad = R Shr 1

		Color (hxClr And $FF0000) Shr 16, (hxClr And $FF00) Shr 8, hxClr And $FF
		X = Rad
		Y = 0
		XChange = 1 - (Rad + Rad)
		YChange = 1
		RadiusError = 0

		While  X >= Y
			If fill <> 0 Then
				Line(CX-X, CY+Y, CX+X, CY+Y)		;used calc'd values to draw
				Line(CX-X, CY-Y, CX+X, CY-Y)		;scan lines from points
				Line(CX-Y, CY+X, CX+Y, CY+X)		;in opposite octants
				Line(CX-Y, CY-X, CX+Y, CY-X)
			Else
				WritePixelFast(CX+X, CY+Y, hxClr )	; {point in octant 1}
				WritePixelFast(CX-X, CY+Y, hxClr )	; {point in octant 4}
				WritePixelFast(CX-X, CY-Y, hxClr )	; {point in octant 5}
				WritePixelFast(CX+X, CY-Y, hxClr )	; {point in octant 8}
				WritePixelFast(CX+Y, CY+X, hxClr )	; {point in octant 2}
				WritePixelFast(CX-Y, CY+X, hxClr )	; {point in octant 3}
				WritePixelFast(CX-Y, CY-X, hxClr )	; {point in octant 6}
				WritePixelFast(CX+Y, CY-X, hxClr )	; {point in octant 7}
			End If
			Y = Y + 1
			RadiusError = RadiusError + YChange
			YChange = YChange + 2
			If RadiusError + RadiusError + XChange > 0 Then
				X = X - 1
				RadiusError = RadiusError + XChange
				XChange = XChange + 2
			End If
		Wend
	End If
End Function





